part of numkeyboard;

typedef KeyboardBuilder = Widget Function(
    BuildContext context, KeyboardController controller, String param);

class Keyboard {
  static JSONMethodCodec _codec = const JSONMethodCodec();
  static KeyboardConfig _currentKeyboard;
  static Map<NumTextInputType, KeyboardConfig> _keyboards = {};
  static KeyboardRootState _root;
  static KeyboardController _keyboardController;
  static GlobalKey<KeyboardPageState> _pageKey;
  static bool isInterceptor = false;

  static String _keyboardParam;
  static Point _keyboardPoint;
  static Timer clearTask;

  static init(KeyboardRootState root, BuildContext context) {
    _root = root;
    interceptorInput();
    MediaQueryData mq = MediaQuery.of(context);
    _read(Orientation.landscape == mq.orientation ? 0 : 1).then((point) {
      _keyboardPoint = point;
    });
    FlutterScreenOrientation.instance().init();
    FlutterScreenOrientation.instance().listenerOrientation((e) {
      var orientation = (e == FlutterScreenOrientation.portraitUp ||
              e == FlutterScreenOrientation.portraitDown)
          ? 1
          : 0;
      _read(orientation).then((point) {
        _keyboardPoint = point;
        _pageKey?.currentState?.updateOnOpen(point);
      });
    });
  }

  static interceptorInput() {
    if (isInterceptor) return;
    if (!(ServicesBinding.instance is KeyboardBiding)) {
      throw Exception('Keyboard can only be used in KeyboardBiding');
    }
    var binding = ServicesBinding.instance as KeyboardBiding;
    var binaryMessenger =
        binding.defaultBinaryMessenger as KeyboardBinaryMessenger;
    binaryMessenger.setMockMessageHandler(
        "flutter/textinput", _textInputHandler);
    isInterceptor = true;
  }

  static Future<Point> _read(int orientation) async {
    // 0:横屏  1:竖屏
    SharedPreferences sp = await SharedPreferences.getInstance();
    double x = 100;
    double y = 50;
    if (orientation == 0) {
      x = sp.getDouble("hx") ?? x;
      y = sp.getDouble("hy") ?? y;
    } else {
      x = sp.getDouble("lx") ?? x;
      y = sp.getDouble("ly") ?? y;
    }
    return Point(x, y);
  }

  static Future<ByteData> _textInputHandler(ByteData data) async {
    var methodCall = _codec.decodeMethodCall(data);
    switch (methodCall.method) {
      case 'TextInput.show':
        if (_currentKeyboard != null) {
          if (clearTask != null) {
            clearTask.cancel();
            clearTask = null;
          }
          openKeyboard();
          return _codec.encodeSuccessEnvelope(null);
        } else {
          if (data != null) {
            return await _sendPlatformMessage("flutter/textinput", data);
          }
        }
        break;
      case 'TextInput.hide':
        if (_currentKeyboard != null) {
          if (clearTask == null) {
            clearTask = new Timer(Duration(milliseconds: 16),
                () => hideKeyboard(animation: true));
          }
          return _codec.encodeSuccessEnvelope(null);
        } else {
          if (data != null) {
            return await _sendPlatformMessage("flutter/textinput", data);
          }
        }
        break;
      case 'TextInput.setEditingState':
        var editingState = TextEditingValue.fromJSON(methodCall.arguments);
        if (_keyboardController != null) {
          _keyboardController.value = editingState;
          return _codec.encodeSuccessEnvelope(null);
        }
        break;
      case 'TextInput.clearClient':
        var isShow = _currentKeyboard != null;
        if (clearTask == null) {
          clearTask = new Timer(
              Duration(milliseconds: 16), () => hideKeyboard(animation: true));
        }
        clearKeyboard();
        if (isShow) {
          return _codec.encodeSuccessEnvelope(null);
        }
        break;
      case 'TextInput.setClient':
        var setInputType = methodCall.arguments[1]['inputType'];
        InputClient client;
        _keyboards.forEach((inputType, keyboardConfig) {
          if (inputType.name == setInputType['name']) {
            client = InputClient.fromJSON(methodCall.arguments);

            _keyboardParam =
                (client.configuration.inputType as NumTextInputType).params;

            clearKeyboard();
            _currentKeyboard = keyboardConfig;
            _keyboardController = KeyboardController(client: client)
              ..addListener(_updateEditingState);
            if (_pageKey != null) {
              _pageKey.currentState?.update();
            }
          }
        });

        if (client != null) {
          await _sendPlatformMessage("flutter/textinput",
              _codec.encodeMethodCall(MethodCall('TextInput.hide')));
          return _codec.encodeSuccessEnvelope(null);
        } else {
          if (clearTask == null) {
            hideKeyboard(animation: false);
          }
          clearKeyboard();
        }
      // break;
    }
    if (data != null) {
      ByteData response = await _sendPlatformMessage("flutter/textinput", data);
      return response;
    }
    return null;
  }

  static void _updateEditingState() {
    var callbackMethodCall = MethodCall("TextInputClient.updateEditingState", [
      _keyboardController.client.connectionId,
      _keyboardController.value.toJSON()
    ]);
    ServicesBinding.instance.defaultBinaryMessenger.handlePlatformMessage(
        "flutter/textinput",
        _codec.encodeMethodCall(callbackMethodCall),
        (data) {});
  }

  static Future<ByteData> _sendPlatformMessage(
      String channel, ByteData message) {
    final Completer<ByteData> completer = Completer<ByteData>();
    ui.window.sendPlatformMessage(channel, message, (ByteData reply) {
      try {
        completer.complete(reply);
      } catch (exception, stack) {
        FlutterError.reportError(FlutterErrorDetails(
          exception: exception,
          stack: stack,
          library: 'services library',
          context:
              ErrorDescription('during a platform message response callback'),
        ));
      }
    });
    return completer.future;
  }

  static addKeyboard(NumTextInputType inputType, KeyboardConfig config) {
    _keyboards[inputType] = config;
  }

  static openKeyboard() {
    if (_root.hasKeyboard && _pageKey != null) return;
    _pageKey = GlobalKey<KeyboardPageState>();
    var tempKey = _pageKey;
    var isUpdate = false;
    _root.setKeyboard((ctx) {
      if (_currentKeyboard != null) {
        if (!isUpdate) {
          isUpdate = true;
        }
        return KeyboardPage(
            key: tempKey,
            point: _keyboardPoint,
            builder: (ctx) {
              return _currentKeyboard?.builder(
                  ctx, _keyboardController, _keyboardParam);
            });
      } else {
        return Container();
      }
    });

    BackButtonInterceptor.add((_, __) {
      Keyboard.sendPerformAction(TextInputAction.done);
      return true;
    }, zIndex: 1, name: 'CustomKeyboard');
  }

  static hideKeyboard({bool animation = true}) {
    if (clearTask != null) {
      if (clearTask.isActive) {
        clearTask.cancel();
      }
      clearTask = null;
    }
    BackButtonInterceptor.removeByName('CustomKeyboard');
    if (_root.hasKeyboard && _pageKey != null) {
      if (animation) {
        _pageKey.currentState.exit();
        Future.delayed(Duration(milliseconds: 116)).then((_) {
          _root.clearKeyboard();
        });
      } else {
        _root.clearKeyboard();
      }
    }
    _pageKey = null;
  }

  static clearKeyboard() {
    _currentKeyboard = null;
    if (_keyboardController != null) {
      _keyboardController.dispose();
      _keyboardController = null;
    }
  }

  static sendPerformAction(TextInputAction action) {
    var callbackMethodCall = MethodCall("TextInputClient.performAction",
        [_keyboardController.client.connectionId, action.toString()]);
    ServicesBinding.instance.defaultBinaryMessenger.handlePlatformMessage(
        "flutter/textinput",
        _codec.encodeMethodCall(callbackMethodCall),
        (data) {});
  }
}

class KeyboardConfig {
  final KeyboardBuilder builder;

  const KeyboardConfig({@required this.builder});
}

class InputClient {
  final int connectionId;
  final TextInputConfiguration configuration;

  const InputClient(
      {@required this.connectionId, @required this.configuration});

  factory InputClient.fromJSON(List<dynamic> encoded) {
    return InputClient(
        connectionId: encoded[0],
        configuration: TextInputConfiguration(
            inputType: NumTextInputType.fromJSON(
              encoded[1]['inputType'],
            ),
            obscureText: encoded[1]['obscureText'],
            autocorrect: encoded[1]['autocorrect'],
            actionLabel: encoded[1]['actionLabel'],
            inputAction: _toTextInputAction(encoded[1]['inputAction']),
            textCapitalization:
                _toTextCapitalization(encoded[1]['textCapitalization']),
            keyboardAppearance:
                _toBrightness(encoded[1]['keyboardAppearance'])));
  }

  static TextInputAction _toTextInputAction(String action) {
    switch (action) {
      case 'TextInputAction.none':
        return TextInputAction.none;
      case 'TextInputAction.unspecified':
        return TextInputAction.unspecified;
      case 'TextInputAction.go':
        return TextInputAction.go;
      case 'TextInputAction.search':
        return TextInputAction.search;
      case 'TextInputAction.send':
        return TextInputAction.send;
      case 'TextInputAction.next':
        return TextInputAction.next;
      case 'TextInputAction.previuos':
        return TextInputAction.previous;
      case 'TextInputAction.continue_action':
        return TextInputAction.continueAction;
      case 'TextInputAction.join':
        return TextInputAction.join;
      case 'TextInputAction.route':
        return TextInputAction.route;
      case 'TextInputAction.emergencyCall':
        return TextInputAction.emergencyCall;
      case 'TextInputAction.done':
        return TextInputAction.done;
      case 'TextInputAction.newline':
        return TextInputAction.newline;
    }
    throw FlutterError('Unknown text input action: $action');
  }

  static TextCapitalization _toTextCapitalization(String capitalization) {
    switch (capitalization) {
      case 'TextCapitalization.none':
        return TextCapitalization.none;
      case 'TextCapitalization.characters':
        return TextCapitalization.characters;
      case 'TextCapitalization.sentences':
        return TextCapitalization.sentences;
      case 'TextCapitalization.words':
        return TextCapitalization.words;
    }

    throw FlutterError('Unknown text capitalization: $capitalization');
  }

  static Brightness _toBrightness(String brightness) {
    switch (brightness) {
      case 'Brightness.dark':
        return Brightness.dark;
      case 'Brightness.light':
        return Brightness.light;
    }

    throw FlutterError('Unknown Brightness: $brightness');
  }
}

class NumTextInputType extends TextInputType {
  final String name;
  final String params;

  const NumTextInputType({this.name, bool signed, bool decimal, this.params})
      : super.numberWithOptions(signed: signed, decimal: decimal);

  @override
  Map<String, dynamic> toJson() {
    return <String, dynamic>{
      'name': name,
      'signed': signed,
      'decimal': decimal,
      'params': params
    };
  }

  @override
  String toString() {
    return '$runtimeType('
        'name: $name, '
        'signed: $signed, '
        'decimal: $decimal)';
  }

  bool operator ==(Object target) {
    if (target is NumTextInputType) {
      if (this.name == target.toString()) {
        return true;
      }
    }
    return false;
  }

  @override
  int get hashCode => this.toString().hashCode;

  factory NumTextInputType.fromJSON(Map<String, dynamic> encoded) {
    return NumTextInputType(
        name: encoded['name'],
        signed: encoded['signed'],
        decimal: encoded['decimal'],
        params: encoded['params']);
  }
}

class KeyboardPage extends StatefulWidget {
  final Widget Function(BuildContext context) builder;
  final Point point;

  const KeyboardPage({@required this.builder, @required this.point, Key key})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return KeyboardPageState(point.x, point.y);
  }
}

class KeyboardPageState extends State<KeyboardPage> {
  Widget _lastBuildWidget;
  bool isClose = false;
  double x = 0;
  double y = 0;

  KeyboardPageState(this.x, this.y);

  @override
  Widget build(BuildContext context) {
    return AnimatedPositioned(
        left: x,
        top: y,
        duration: Duration.zero,
        child: GestureDetector(
          onPanUpdate: (event) {
            setState(() {
              x += event.delta.dx;
              y += event.delta.dy;
            });
          },
          onPanEnd: (event) {
            MediaQueryData mq = MediaQuery.of(context);
            _save(mq.orientation == Orientation.landscape ? 0 : 1, Point(x, y));
          },
          child: IntrinsicHeight(
            child: Builder(
              builder: (ctx) {
                var result = widget.builder(ctx);
                if (result != null) {
                  _lastBuildWidget = result;
                }
                return Card(
                  color: Color(0xFFEEEEEE),
                  child: Container(
                    child: _lastBuildWidget,
                  ),
                );
              },
            ),
          ),
        ));
  }

  exit() {
    isClose = true;
  }

  update() {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      setState(() => {});
    });
  }

  updateOnOpen(Point point) {
    setState(() {
      x = point.x;
      y = point.y;
    });
  }

  Future _save(int orientation, Point point) async {
    // 0:横屏  1:竖屏
    SharedPreferences sp = await SharedPreferences.getInstance();
    if (orientation == 0) {
      sp.setDouble("hx", point.x);
      sp.setDouble("hy", point.y);
    } else {
      sp.setDouble("lx", point.x);
      sp.setDouble("ly", point.y);
    }
  }
}
