part of numkeyboard;

class KeyboardRoot extends StatefulWidget {
  final Widget child;
  final TextDirection textDirection;

  const KeyboardRoot(
      {Key key, @required this.child, this.textDirection = TextDirection.ltr})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => KeyboardRootState();
}

class KeyboardRootState extends State<KeyboardRoot> {
  WidgetBuilder _keyboardBuilder;

  bool get hasKeyboard => _keyboardBuilder != null;

  @override
  Widget build(BuildContext context) {
    var data = MediaQuery.maybeOf(context);
    if (data == null) {
      data = MediaQueryData.fromWindow(WidgetsBinding.instance.window);
    }
    return MediaQuery(
        data: data.copyWith(viewInsets: data.viewInsets),
        child: Builder(builder: (context) {
          Keyboard.init(this, context);
          List<Widget> children = [widget.child];
          if (_keyboardBuilder != null) {
            children.add(Builder(
              builder: _keyboardBuilder,
            ));
          }
          return Directionality(
              textDirection: widget.textDirection,
              child: Stack(
                children: children,
              ));
        }));
  }

  setKeyboard(WidgetBuilder builder) {
    this._keyboardBuilder = builder;
    setState(() {});
  }

  clearKeyboard() {
    if (this._keyboardBuilder != null) {
      this._keyboardBuilder = null;
      setState(() {});
    }
  }
}
