library numkeyboard;

import 'dart:async';
import 'dart:core';
import 'dart:math';
import 'dart:ui';
import 'dart:ui' as ui;

import 'package:back_button_interceptor/back_button_interceptor.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screen_orientation/flutter_screen_orientation.dart';
import 'package:shared_preferences/shared_preferences.dart';

part 'keyboards/keyboard_binary_messenger.dart';

part 'keyboards/keyboard_binding.dart';

part 'keyboards/keyboard_controller.dart';

part 'keyboards/keyboard_manager.dart';

part 'keyboards/keyboard_root.dart';

class NumKeyboard extends StatelessWidget {
  static const NumTextInputType inputType =
      const NumTextInputType(name: 'NumKeyboard');

  final KeyboardController controller;

  const NumKeyboard({@required this.controller});

  static register() {
    Keyboard.addKeyboard(
        NumKeyboard.inputType,
        KeyboardConfig(
            builder: (context, controller, params) =>
                NumKeyboard(controller: controller)));
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 220,
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Container(
                width: 45,
                height: 40,
                child: IconButton(
                  color: Color(0xFF363636),
                  icon: Icon(
                    Icons.settings_ethernet,
                    size: 18,
                  ),
                  onPressed: () {},
                ),
              ),
              Container(
                width: 45,
                height: 40,
                child: IconButton(
                  color: Color(0xFF363636),
                  icon: Icon(
                    Icons.highlight_off,
                    size: 18,
                  ),
                  onPressed: () {
                    controller.doneAction();
                  },
                ),
              )
            ],
          ),
          Padding(
            padding: EdgeInsets.symmetric(horizontal: 4.0),
            child: Divider(height: 1, color: Color(0xFFDFDFDF)),
          ),
          Container(
            padding: EdgeInsets.all(4.0),
            width: 210,
            height: 150,
            child: MediaQuery.removePadding(
              context: context,
              removeTop: true,
              child: GridView.count(
                crossAxisCount: 3,
                childAspectRatio: 2,
                mainAxisSpacing: 4,
                crossAxisSpacing: 4,
                children: [
                  button('1'),
                  button('2'),
                  button('3'),
                  button('4'),
                  button('5'),
                  button('6'),
                  button('7'),
                  button('8'),
                  button('9'),
                  button('.'),
                  button('0'),
                  TextButton.icon(
                      onPressed: () {
                        controller.deleteOne();
                      },
                      onLongPress: () {
                        controller.clear();
                      },
                      style: ButtonStyle(
                        backgroundColor:
                            MaterialStateProperty.resolveWith((states) {
                          if (states.contains(MaterialState.pressed)) {
                            return Colors.blue;
                          }
                          return Color(0xFFD2D2D2);
                        }),
                      ),
                      icon: Container(
                          width: 0,
                          child: Icon(Icons.backspace_rounded,
                              size: 18, color: Color(0xFF363636))),
                      label: Container(width: 10))
                ],
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget button(String title, {String value}) {
    return TextButton(
      onPressed: () {
        controller.addText(value ?? title);
      },
      child: Text(title),
      style: ButtonStyle(
        backgroundColor: MaterialStateProperty.resolveWith((states) {
          if (states.contains(MaterialState.pressed)) {
            return Colors.blue;
          }
          return Color(0xFFFFFFFF);
        }),
        foregroundColor: MaterialStateProperty.resolveWith((states) {
          if (states.contains(MaterialState.pressed)) {
            return Colors.white;
          }
          return Color(0xFF363636);
        }),
      ),
    );
  }
}
